@@ -46,8 +46,21 @@ from spellbot.constants import (
     DEFAULT_GAME_SIZE,
     INVITE_LINK,
     THUMB_URL,
+    VOTE_LINK,
+)
+from spellbot.data import (
+    Channel,
+    Data,
+    Event,
+    Game,
+    Report,
+    Server,
+    Tag,
+    Team,
+    User,
+    UserPoints,
+    UserTeam,
 )
-from spellbot.data import Channel, Data, Event, Game, Server, Tag, User
 from spellbot.reactions import safe_clear_reactions, safe_remove_reaction
 
 load_dotenv()
@@ -255,8 +268,8 @@ class SpellBot(discord.Client):
 
     def __init__(
         self,
-        token: str = "",
-        auth: Optional[str] = "",
+        token: Optional[str] = None,
+        auth: Optional[str] = None,
         db_url: str = DEFAULT_DB_URL,
         log_level: Union[int, str] = logging.ERROR,
         mock_games: bool = False,
@@ -285,8 +298,8 @@ class SpellBot(discord.Client):
         if not loop:
             loop = asyncio.get_event_loop()
         super().__init__(loop=loop)
-        self.token = token
-        self.auth = auth
+        self.token = token or ""
+        self.auth = auth or ""
         self.mock_games = mock_games
 
         # We have to make sure that DB_DIR exists before we try to create
@@ -862,7 +875,9 @@ class SpellBot(discord.Client):
             " <https://github.com/lexicalunit/spellbot/issues>!\n"
         )
         usage += "\n"
-        usage += f"Add SpellBot to your Discord: <{INVITE_LINK}>\n"
+        usage += f"🔗 Add SpellBot to your Discord: <{INVITE_LINK}>\n"
+        usage += "\n"
+        usage += f"👍 Give SpellBot a vote on top.gg: <{VOTE_LINK}>\n"
         usage += "\n"
         usage += (
             "💜 You can help keep SpellBot running by supporting me on Ko-fi! "
@@ -897,6 +912,8 @@ class SpellBot(discord.Client):
             "\n"
             f"[🔗 Add SpellBot to your Discord!]({INVITE_LINK})\n"
             "\n"
+            f"[👍 Give SpellBot a vote on top.gg!]({VOTE_LINK})\n"
+            "\n"
             "💜 Help keep SpellBot running by "
             "[supporting me on Ko-fi!](https://ko-fi.com/Y8Y51VTHZ)"
         )
@@ -1208,6 +1225,158 @@ class SpellBot(discord.Client):
             session, prefix, params, message, create_new_game=False, use_queue=False
         )
 
+    async def _verify_command_fest_report(
+        self,
+        session: Session,
+        prefix: str,
+        params: List[str],
+        message: discord.Message,
+        game: Game,
+    ) -> bool:
+        server = self.ensure_server_exists(session, message.channel.guild.id)
+        mentioned_users = []
+        for mentioned in message.mentions:
+            mentioned_user = self.ensure_user_exists(session, mentioned)
+            mentioned_users.append(mentioned_user)
+
+        points = []
+        for param in params[1:]:
+            if param.isdigit():
+                points.append(int(param))
+
+        if len(mentioned_users) != game.size:
+            await message.channel.send(
+                s(
+                    "report_incomplete",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            return False
+
+        if len(mentioned_users) == len(points):
+            for mentioned_user, pointage in zip(mentioned_users, points):
+                user_points = (
+                    session.query(UserPoints)
+                    .filter_by(
+                        user_xid=mentioned_user.xid,
+                        guild_xid=server.guild_xid,
+                        game_id=game.id,
+                    )
+                    .one_or_none()
+                )
+                if not user_points:
+                    user_points = UserPoints(
+                        user_xid=mentioned_user.xid,
+                        guild_xid=server.guild_xid,
+                        game_id=game.id,
+                        points=pointage,
+                    )
+                    session.add(user_points)
+                else:
+                    user_points.points = pointage
+            session.commit()
+        else:
+            await message.channel.send(
+                s("report_wrong", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return False
+
+        return True
+
+    @command(allow_dm=False, aliases=["results"])
+    async def report(
+        self, session: Session, prefix: str, params: List[str], message: discord.Message
+    ) -> None:
+        """
+        Report your results on a finished game.
+        & <Game ID> ...
+        """
+        if len(params) < 2:
+            await message.channel.send(
+                s("report_no_params", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        req = params[0]
+        report_str = " ".join(params[1:])
+
+        if len(report_str) >= 255:
+            await message.channel.send(
+                s("report_too_long", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        game: Optional[Game] = None
+        if req.lower().startswith("sb") and req[2:].isdigit():
+            game_id = int(req[2:])
+            game = session.query(Game).filter(Game.id == game_id).one_or_none()
+        elif req.isdigit():
+            game_id = int(req)
+            game = session.query(Game).filter(Game.id == game_id).one_or_none()
+        elif re.match(r"^[\w-]*$", req):  # perhaps it's a spellbot game id
+            game = session.query(Game).filter(Game.url.ilike(f"%{req}")).one_or_none()
+
+        if not game:
+            await message.channel.send(
+                s("report_no_game", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        if not game.status == "started":
+            await message.channel.send(
+                s(
+                    "report_not_started",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            return
+
+        # TODO: This reporting logic is specific to CommandFest, it would be nice
+        #       to refactor this to be more flexible after the event.
+        verified = await self._verify_command_fest_report(
+            session=session, prefix=prefix, params=params, message=message, game=game
+        )
+        if not verified:
+            return
+
+        report = Report(game_id=game.id, report=report_str)
+        session.add(report)
+        session.commit()
+        await message.channel.send(
+            s("report", reply=f"<@{cast(discord.User, message.author).id}>")
+        )
+
+    @command(allow_dm=False, admin_only=False)
+    async def points(
+        self, session: Session, prefix: str, params: List[str], message: discord.Message
+    ) -> None:
+        """
+        Get your total points on this server.
+        """
+        user = self.ensure_user_exists(session, message.author)
+        server = self.ensure_server_exists(session, message.channel.guild.id)
+        points = user.points(server.guild_xid)
+
+        await message.channel.send(
+            s(
+                "points",
+                reply=f"<@{cast(discord.User, message.author).id}>",
+                points=points,
+            )
+        )
+
+        if is_admin(message.channel, message.author):
+            team_points = Team.points(session, message.channel.guild.id)
+            for team, team_points in team_points.items():
+                await message.channel.send(
+                    s(
+                        "points_team",
+                        reply=f"<@{cast(discord.User, message.author).id}>",
+                        team=team,
+                        points=team_points,
+                    )
+                )
+
     @command(allow_dm=False, admin_only=True)
     async def event(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
@@ -1710,6 +1879,94 @@ class SpellBot(discord.Client):
             s("power", reply=f"<@{cast(discord.User, message.author).id}>", power=power_i)
         )
 
+    @command(allow_dm=True)
+    async def team(
+        self, session: Session, prefix: str, params: List[str], message: discord.Message
+    ) -> None:
+        """
+        Set or get your team on this server. To get your team name, run this command with
+        no parameters.
+        & [team-name]
+        """
+        server = self.ensure_server_exists(session, message.channel.guild.id)
+        if not server.teams:
+            await message.channel.send(
+                s("team_none", reply=f"<@{cast(discord.User, message.author).id}>")
+            )
+            return
+
+        user = self.ensure_user_exists(session, message.author)
+        if not params:
+            user_team = (
+                session.query(UserTeam)
+                .filter_by(user_xid=user.xid, guild_xid=server.guild_xid)
+                .one_or_none()
+            )
+
+            if not user_team:
+                await message.channel.send(
+                    s("team_not_set", reply=f"<@{cast(discord.User, message.author).id}>")
+                )
+                return
+
+            team = session.query(Team).filter_by(id=user_team.team_id).one_or_none()
+            if not team:
+                session.delete(user_team)
+                await message.channel.send(
+                    s("team_gone", reply=f"<@{cast(discord.User, message.author).id}>")
+                )
+                return
+
+            await message.channel.send(
+                s(
+                    "team_yours",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                    team=team.name,
+                )
+            )
+            return
+
+        team_request = params[0]
+        team_found: Optional[Team] = None
+        for team in server.teams:
+            if team_request.lower() != team.name.lower():
+                continue
+            team_found = team
+            break
+
+        if not team_found:
+            teams = ", ".join(sorted(team.name for team in server.teams))
+            await message.channel.send(
+                s(
+                    "team_not_found",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                    teams=teams,
+                ),
+            )
+            return
+
+        user_team = (
+            session.query(UserTeam)
+            .filter_by(user_xid=user.xid, guild_xid=server.guild_xid)
+            .one_or_none()
+        )
+        if user_team:
+            user_team.team_id = team_found.id
+        else:
+            user_team = UserTeam(
+                user_xid=user.xid, guild_xid=server.guild_xid, team_id=team_found.id
+            )
+            session.add(user_team)
+        session.commit()
+
+        await message.channel.send(
+            s(
+                "team",
+                reply=f"<@{cast(discord.User, message.author).id}>",
+                team=team_found.name,
+            ),
+        )
+
     @command(allow_dm=False)
     async def spellbot(
         self, session: Session, prefix: str, params: List[str], message: discord.Message
@@ -1723,6 +1980,7 @@ class SpellBot(discord.Client):
         * `prefix <string>`: Set SpellBot's command prefix for text channels.
         * `links <string>`: Set the privacy level for generated SpellTable links.
         * `expire <number>`: Set the number of minutes before pending games expire.
+        * `teams <list>`: Sets the teams available on this server.
         * `help`: Get detailed usage help for SpellBot.
         & <subcommand> [subcommand parameters]
         """
@@ -1754,6 +2012,8 @@ class SpellBot(discord.Client):
             await self.spellbot_config(session, server, params[1:], message)
         elif command == "links":
             await self.spellbot_links(session, server, params[1:], message)
+        elif command == "teams":
+            await self.spellbot_teams(session, server, params[1:], message)
         else:
             await message.channel.send(
                 s(
@@ -1943,6 +2203,48 @@ class SpellBot(discord.Client):
             )
         )
 
+    async def spellbot_teams(
+        self,
+        session: Session,
+        server: Server,
+        params: List[str],
+        message: discord.Message,
+    ) -> None:
+        if not params:
+            await message.channel.send(
+                s(
+                    "spellbot_teams_none",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            return
+
+        if len(params) < 2:
+            await message.channel.send(
+                s(
+                    "spellbot_teams_too_few",
+                    reply=f"<@{cast(discord.User, message.author).id}>",
+                )
+            )
+            return
+
+        # blow away any existing old teams
+        for team in server.teams:
+            session.delete(team)
+        session.commit()
+
+        # then create new ones
+        server.teams = [Team(name=name) for name in set(params)]
+        session.commit()
+
+        await message.channel.send(
+            s(
+                "spellbot_teams",
+                reply=f"<@{cast(discord.User, message.author).id}>",
+                teams=", ".join(sorted(team.name for team in server.teams)),
+            )
+        )
+
     async def spellbot_config(
         self,
         session: Session,
@@ -1962,6 +2264,9 @@ class SpellBot(discord.Client):
             channels_str = "all"
         embed.add_field(name="Active channels", value=channels_str)
         embed.add_field(name="Links", value=server.links.title())
+        if server.teams:
+            teams_str = ", ".join(sorted(team.name for team in server.teams))
+            embed.add_field(name="Teams", value=teams_str)
         embed.color = discord.Color(0x5A3EFD)
         embed.set_footer(text=f"Config for Guild ID: {server.guild_xid}")
         await message.channel.send(embed=embed)
