@@ -50,6 +50,7 @@ class Server(Base):
     links = Column(String(10), nullable=False, server_default=text("'public'"))
     games = relationship("Game", back_populates="server", uselist=True)
     channels = relationship("Channel", back_populates="server", uselist=True)
+    teams = relationship("Team", back_populates="server", uselist=True)
 
     def bot_allowed_in(self, channel_xid: int) -> bool:
         return not self.channels or any(
@@ -96,10 +97,40 @@ class Server(Base):
                 "prefix": self.prefix,
                 "expire": self.expire,
                 "channels": [channel.channel_xid for channel in self.channels],
+                "teams": [team.id for team in self.teams],
             }
         )
 
 
+class Team(Base):
+    __tablename__ = "teams"
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    guild_xid = Column(
+        BigInteger,
+        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
+        nullable=False,
+        index=True,
+    )
+    name = Column(String(50), nullable=False)
+    server = relationship("Server", back_populates="teams")
+
+    @classmethod
+    def points(cls, session: Session, guild_xid: int) -> dict:
+        rows = (
+            session.query(Team.name, func.sum(UserPoints.points))
+            .select_from(User)
+            .join(UserTeam)
+            .join(UserPoints)
+            .join(Team)
+            .group_by(Team.name)
+            .all()
+        )
+        results = {}
+        for row in rows:
+            results[row[0]] = row[1]
+        return results
+
+
 class Channel(Base):
     __tablename__ = "channels"
     channel_xid = Column(BigInteger, primary_key=True, nullable=False)
@@ -155,6 +186,15 @@ class User(Base):
             return True
         return False
 
+    def points(self, guild_xid) -> int:
+        session = Session.object_session(self)
+        results = (
+            session.query(func.sum(UserPoints.points))
+            .filter_by(user_xid=self.xid, guild_xid=guild_xid)
+            .scalar()
+        )
+        return results or 0
+
     def to_json(self) -> dict:
         return {
             "xid": self.xid,
@@ -166,6 +206,46 @@ class User(Base):
         }
 
 
+class UserTeam(Base):
+    __tablename__ = "user_teams"
+    user_xid = Column(
+        BigInteger,
+        ForeignKey("users.xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+    )
+    guild_xid = Column(
+        BigInteger,
+        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+    )
+    team_id = Column(Integer, ForeignKey("teams.id", ondelete="CASCADE"), nullable=False)
+
+
+class UserPoints(Base):
+    __tablename__ = "user_points"
+    user_xid = Column(
+        BigInteger,
+        ForeignKey("users.xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+    )
+    guild_xid = Column(
+        BigInteger,
+        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+    )
+    game_id = Column(
+        Integer,
+        ForeignKey("games.id", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+    )
+    points = Column(Integer, nullable=False)
+
+
 class Tag(Base):
     __tablename__ = "tags"
     id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
@@ -220,6 +300,7 @@ class Game(Base):
     tags = relationship("Tag", secondary=games_tags, back_populates="games", uselist=True)
     server = relationship("Server", back_populates="games")
     event = relationship("Event", back_populates="games")
+    reports = relationship("Report", back_populates="game", uselist=True)
 
     @property
     def power(self) -> Optional[float]:
@@ -370,6 +451,17 @@ class Game(Base):
         return embed
 
 
+class Report(Base):
+    __tablename__ = "reports"
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    game_id = Column(
+        Integer, ForeignKey("games.id", ondelete="CASCADE"), nullable=False, index=True
+    )
+    report = Column(String(255), nullable=False)
+    created_at = Column(DateTime, nullable=False, default=datetime.utcnow)
+    game = relationship("Game", back_populates="reports")
+
+
 def create_all(connection: Connection, db_url: str) -> None:
     config = alembic.config.Config(str(ALEMBIC_INI))
     config.set_main_option("script_location", str(VERSIONS_DIR))
